@@ -1,21 +1,24 @@
 import itertools
-import re
 from typing import Callable, Collection, Dict, List, Optional
 
 import cytoolz as tz
-import nltk
 import torch
+from meerkat.tools.lazy_loader import LazyLoader
 from transformers import (
     AutoModel,
+    AutoModelForQuestionAnswering,
     AutoModelForSeq2SeqLM,
     AutoModelForSequenceClassification,
     AutoTokenizer,
 )
 
-from robustnessgym.core.dataset import Dataset
 from robustnessgym.core.metrics import compute_metric
+from robustnessgym.core.slice import SliceDataPanel as DataPanel
 from robustnessgym.tasks.task import Task
 
+ludwig_api = LazyLoader("ludwig.api")
+nltk = LazyLoader("nltk")
+
 
 class Model:
     def __init__(
@@ -65,13 +68,15 @@ class Model:
             if torch.cuda.is_available():
                 self.device = "cuda:0"
 
+        self.is_classifier = is_classifier
+
     def to(self, device: str):
         self.device = device
         return self.model.to(device)
 
     def __call__(
         self,
-        dataset: Dataset,
+        dataset: DataPanel,
         input_columns: List[str],
         output_columns: List[str],
         batch_size: int = 32,
@@ -87,7 +92,7 @@ class Model:
             batch_size,
             coerce_fn,
             *args,
-            **kwargs
+            **kwargs,
         )
 
     @classmethod
@@ -130,7 +135,7 @@ class Model:
 
     def evaluate(
         self,
-        dataset: Dataset,
+        dataset: DataPanel,
         input_columns: List[str],
         output_columns: List[str],
         batch_size: int = 32,
@@ -195,11 +200,17 @@ class HuggingfaceModel(Model):
                 self.model = AutoModelForSequenceClassification.from_pretrained(
                     self.identifier
                 )
+            elif self.task.identifier == "ExtractiveQuestionAnswering":
+                self.model = AutoModelForQuestionAnswering.from_pretrained(
+                    self.identifier
+                )
             else:
                 self.model = AutoModelForSeq2SeqLM.from_pretrained(self.identifier)
 
         self.task = task
 
+        self.is_classifier = is_classifier
+
         # Move the model to device
         self.to(self.device)
 
@@ -207,7 +218,7 @@ class HuggingfaceModel(Model):
         # Create the required outputs
         output_dict = {k: None for k in self.outputs}
 
-        if self.task.classification():
+        if self.is_classifier:
             # Run the model on the input_batch
             # TODO(karan): allow outputs to generically contain side information (
             #  embeddings, attention, etc.)
@@ -248,7 +259,10 @@ class HuggingfaceModel(Model):
     def encode_batch(self, batch: Dict[str, List], columns: Collection[str], **kwargs):
         # TODO(karan): Automatically writing this encoder for a variety of tasks
         return self.tokenizer(
-            *[batch[key] for key in columns], truncation=True, padding=True, **kwargs
+            *[list(batch[key]) for key in columns],
+            truncation=True,
+            padding=True,
+            **kwargs,
         )
 
     def predict_batch(self, batch: Dict[str, List], input_columns: Collection[str]):
@@ -266,7 +280,7 @@ class HuggingfaceModel(Model):
 
     def evaluate(
         self,
-        dataset: Dataset,
+        dataset: DataPanel,
         input_columns: List[str],
         output_columns: List[str],
         batch_size: int = 32,
@@ -280,7 +294,7 @@ class HuggingfaceModel(Model):
         dataset.reset_format()
         dataset.set_format(columns=input_columns + output_columns)
 
-        # TODO(karan): check that the Dataset conforms to the task definition
+        # TODO(karan): check that the DataPanel conforms to the task definition
         # TODO(karan): figure out how the output_columns will be used by the metrics
         pass
 
@@ -308,7 +322,7 @@ class HuggingfaceModel(Model):
 
             # TODO(karan): general version for non-classification problems
             # TODO(karan): move this to the right device
-            if self.task.classification():
+            if self.is_classifier:
                 target_dict = tz.valmap(lambda v: torch.tensor(v), target_dict)
 
             # TODO(karan): incremental metric computation here
@@ -317,7 +331,7 @@ class HuggingfaceModel(Model):
             targets.append(target_dict)
 
         # Consolidate the predictions and targets
-        if self.task.classification():
+        if self.is_classifier:
             # TODO(karan): Need to store predictions and outputs from the model
             predictions = tz.merge_with(lambda v: torch.cat(v).to("cpu"), *predictions)
             targets = tz.merge_with(lambda v: torch.cat(v).to("cpu"), *targets)
@@ -334,10 +348,10 @@ class HuggingfaceModel(Model):
 
         # Assumes classification, so the output_columns contains a single key for the
         # label
-        if self.task.classification():
+        if self.is_classifier:
             assert len(output_columns) == 1  # , "Only supports classification."
             num_classes = self.task.output_schema.features[
-                list(self.task.output_schema.keys())[0]
+                list(self.task.output_schema.columns)[0]
             ].num_classes
 
         labels = targets[list(targets.keys())[0]]
@@ -363,7 +377,25 @@ class HuggingfaceModel(Model):
         return evaluation_dict
 
 
-def format_summary(x: str) -> str:
-    """Format summary text for computing rouge."""
-    re.sub("<n>", "", x)  # remove pegasus newline char
-    return "\n".join(nltk.sent_tokenize(x))
+class LudwigModel(Model):
+    def load(
+        self,
+        model_dir: str,
+    ):
+        self.ludwig_model = ludwig_api.LudwigModel.load(model_dir)
+
+    def evaluate(
+        self,
+        dataset: DataPanel,
+        batch_size: int = 128,
+        collect_overall_stats: bool = True,
+        collect_predictions: bool = True,
+    ):
+        eval_stats, predictions, _ = self.ludwig_model.evaluate(
+            dataset=dataset[:],
+            batch_size=batch_size,
+            collect_overall_stats=collect_overall_stats,
+            collect_predictions=collect_predictions,
+        )
+
+        return eval_stats, predictions
