@@ -1,6 +1,7 @@
 from __future__ import annotations
 
 import json
+import logging
 import pathlib
 from typing import Callable, Collection, Dict, List, Optional, Sequence, Union
 
@@ -17,27 +18,32 @@ from robustnessgym.core.constants import (
     GENERIC,
     SUBPOPULATION,
 )
-from robustnessgym.core.metrics import compute_metric
+from robustnessgym.core.identifier import Identifier
+from robustnessgym.core.metrics import compute_metric, get_metric
 from robustnessgym.core.model import Model
-from robustnessgym.core.report import (
+from robustnessgym.core.slice import SliceDataPanel as DataPanel
+from robustnessgym.core.tools import persistent_hash
+from robustnessgym.core.version import SemanticVersionerMixin
+from robustnessgym.report.report import (
     ClassDistributionColumn,
     NumericColumn,
     Report,
     ScoreColumn,
 )
-from robustnessgym.core.slice import Slice
-from robustnessgym.core.tools import persistent_hash
-from robustnessgym.core.version import SemanticVersionerMixin
+from robustnessgym.tasks.schema import Schema
 from robustnessgym.tasks.task import Task
 
+logger = logging.getLogger(__name__)
+
 
-# TODO(karan): make the TestBench hashable
 class TestBench(SemanticVersionerMixin):
+    """Class for test benches in Robustness Gym."""
+
     def __init__(
         self,
-        identifier: str,
+        identifier: Union[str, Identifier],
         task: Task = None,
-        slices: Collection[Slice] = None,
+        slices: Collection[DataPanel] = None,
         version: str = "0.0.1",
         dataset_id: str = None,
         class_names: Collection[str] = None,
@@ -51,9 +57,11 @@ class TestBench(SemanticVersionerMixin):
 
         # Set the task
         self.task = task
+        if task is None:
+            self.task = Task()
 
         # Create the collection of slices
-        self.slices = set()
+        self._slices = set()
         self.slice_identifiers = set()
         self._slice_table = {}
 
@@ -61,6 +69,9 @@ class TestBench(SemanticVersionerMixin):
         if slices:
             self.add_slices(slices)
 
+        # The testbench has calculators
+        self.calculators = set()
+
         # The testbench internally tracks metrics
         self.metrics = {}
 
@@ -72,22 +83,96 @@ class TestBench(SemanticVersionerMixin):
 
         self.class_names = class_names
 
-    def digest(self) -> str:
+    @property
+    def slices(self):
+        """Slices in the testbench."""
+        return self._slices
+
+    def __repr__(self):
+        return f"TestBench[{self.identifier}](slices={len(self.slices)})"
+
+    def _digest(self) -> str:
         return json.dumps([str(sl) for sl in self.slices])
 
     @classmethod
-    def for_dataset(
-        cls, dataset: str, task: Optional[Union[str, Task]] = None, version: str = None
+    def from_dataset(
+        cls,
+        dp: DataPanel,
+        input_columns: List[str],
+        output_columns: List[str],
+        # prediction_columns: List[str],
+        # metrics: List[str],
+    ) -> TestBench:
+        """Create a TestBench from a dp."""
+        # Define the task
+        task = Task(
+            # Identifier
+            Identifier("Task", dp=str(dp.identifier)),
+            # Input and output schemas
+            *Schema.for_dataset(dp, input_columns, output_columns),
+        )
+
+        # Create the testbench
+        testbench = TestBench(
+            identifier=Identifier("TestBench", dp=str(dp.identifier)),
+            task=task,
+            slices=[dp],
+        )
+
+        # testbench.set_single_dataset_mode()
+        # testbench.set_prediction_columns(prediction_columns)
+
+        return testbench
+
+    def set_prediction_columns(self, prediction_columns: List[str]):
+        """Set the list of columns that act as prediction columns."""
+        self.prediction_columns = prediction_columns
+
+    def set_single_dataset_mode(self):
+        """All slices must be derived from the root dataset in single dataset
+        mode."""
+        self.single_dataset_mode = True
+
+    def add_calculators(self, calculators: List[Union[str, Callable]]):
+        """Add a list of calculators."""
+        for calc in calculators:
+            if isinstance(calc, str):
+                self.calculators.add(get_metric(calc))
+            elif isinstance(calc, Callable):
+                self.calculators.add(calc)
+            else:
+                continue
+
+    def add_model(
+        self,
+        model: Model = None,
+        prediction_columns: List[str] = None,
+        identifier: Union[str, Identifier] = None,
     ):
-        """Create a test bench for a dataset.
 
-        Args:
-            dataset:
-            task:
-            version:
+        if prediction_columns is not None:
+            assert model is None, "`model` must be None."
+            assert (
+                identifier is not None
+            ), "A model `identifier` must be included with `prediction_columns`."
 
-        Returns:
-        """
+            self.metrics[identifier] = {}
+            for sl in self.slices:
+                self.metrics[identifier][sl.identifier] = {}
+                for calc in self.calculators:
+                    self.metrics[identifier][sl.identifier][calc.__name__] = calc(
+                        *[sl[col] for col in prediction_columns],
+                        *[sl[col] for col in self.task.output_schema.columns],
+                    )
+
+    @classmethod
+    def for_dataset(
+        cls,
+        dataset: str,
+        task: Optional[Union[str, Task]] = None,
+        version: str = None,
+    ):
+        """Create a test bench for a dataset."""
 
         inferred_task = None
         if task is not None:
@@ -96,7 +181,7 @@ class TestBench(SemanticVersionerMixin):
             # Check that the inferred task matches the task argument
             if task is not None and task != inferred_task:
                 raise AssertionError(
-                    f"Dataset {dataset} is only compatible with {inferred_task}, "
+                    f"DataPanel {dataset} is only compatible with {inferred_task}, "
                     f"not {task}."
                 )
 
@@ -112,6 +197,7 @@ class TestBench(SemanticVersionerMixin):
         task: Union[str, Task],
         version: str = None,
     ):
+        """Create a testbench for a task."""
         return TestBench(
             identifier=f"{task}-{version}",
             task=task,
@@ -167,24 +253,28 @@ class TestBench(SemanticVersionerMixin):
 
         self.ident_mapping = ident_mapping
 
-    def add_slices(self, slices: Collection[Slice]):
+    def add_slices(self, slices: Collection[DataPanel]) -> None:
         """Add slices to the testbench.
 
         Args:
             slices: collection of Slice objects
-
-        Returns:
         """
-        if isinstance(slices, Slice):
+        if isinstance(slices, DataPanel):
             slices = [slices]
 
-        # Only add slices that aren't already present in the testbench and have
-        # non-zero length
+        # Add slices
         for sl in slices:
-            if sl.identifier not in self.slice_identifiers and len(sl) > 0:
-                self.slices.add(sl)
-                self.slice_identifiers.add(sl.identifier)
-                self._slice_table[sl.identifier] = sl
+            if not isinstance(sl, DataPanel) and isinstance(sl, DataPanel):
+                # The slice is a DataPanel
+                sl = DataPanel(sl)
+
+            if isinstance(sl, DataPanel):
+                if sl.identifier not in self.slice_identifiers and len(sl) > 0:
+                    # Add slices that aren't already present in the testbench and have
+                    # non-zero length
+                    self.slices.add(sl)
+                    self.slice_identifiers.add(sl.identifier)
+                    self._slice_table[sl.identifier] = sl
 
     def evaluate(
         self,
@@ -216,14 +306,14 @@ class TestBench(SemanticVersionerMixin):
             # Set the schema using the task
             # TODO Is the remapping required when not using a task
             self.set_schema("task")
-            input_columns = self.task.input_schema.keys()
-            output_columns = self.task.output_schema.keys()
+            input_columns = self.task.input_schema.columns
+            output_columns = self.task.output_schema.columns
 
         # TODO(karan): Uncomment and fix this assert on the type of outputs that
         #  model(..) returns
         # # Grab 2 examples from the first slice, run it through the model and check
         # that the output is a dictionary
-        # output = model(dataset=Dataset.from_batch(self.slices[0][:2]),
+        # output = model(dataset=DataPanel.from_batch(self.slices[0][:2]),
         #                input_keys=self.task.input_schema.keys(),
         #                output_keys=self.task.output_schema.keys(),
         #                batch_size=2,
@@ -240,7 +330,7 @@ class TestBench(SemanticVersionerMixin):
         # Run the model on all the slices
         # TODO(karan): For slices that are subpopulations, the same example can be in
         #  multiple slices
-        #  and will be run through the model multiple times. Create a UnionSlice?
+        #  and will be run through the model multiple times. Create a UnionDataPanel?
         for sl in tqdm(self.slices):
             if sl.identifier not in self.metrics[model.identifier]:
                 # Evaluate on the slice
@@ -263,7 +353,8 @@ class TestBench(SemanticVersionerMixin):
         num_classes=None,
         metrics: List[str] = None,
     ) -> Dict:
-        """Compute and cache metrics for pre-computed model predictions
+        """Compute and cache metrics for pre-computed model predictions.
+
         Args:
             model: Model or model id
             predictions: Map from slice id to sequence or torch Tensor of predictions
@@ -273,7 +364,9 @@ class TestBench(SemanticVersionerMixin):
             does not have associated task.
             num_classes (optional): number of classes. Required if testbench does not
             have associated task.
-        Returns: computed metrics
+
+        Returns:
+            computed metrics
         """
 
         if self.task is None:
@@ -292,9 +385,9 @@ class TestBench(SemanticVersionerMixin):
                     "'metrics' is required if testbench does not have associated task."
                 )
         else:
-            output_columns = self.task.output_schema.keys()
+            output_columns = self.task.output_schema.columns
             num_classes = self.task.output_schema.features[
-                list(self.task.output_schema.keys())[0]
+                list(self.task.output_schema.columns)[0]
             ].num_classes
             if self.task.classification():
                 assert len(output_columns) == 1  # , "Only supports classification."
@@ -333,7 +426,8 @@ class TestBench(SemanticVersionerMixin):
         return evaluation_dict
 
     def add_metrics(self, model: Union[Model, str], metrics: Dict[str, float]):
-        """Cache pre-computed metrics for model
+        """Cache pre-computed metrics for model.
+
         Args:
             model: Model or model id.
             metrics: map from metric name to value
@@ -347,12 +441,14 @@ class TestBench(SemanticVersionerMixin):
         model: Union[Model, str],
         metric_ids: List[str] = None,
     ) -> Report:
-        """Generate report from cached metrics for a model
+        """Generate report from cached metrics for a model.
+
         Args:
             model: Model or model id. Metrics must have already been computed for
             this model.
             metric_ids (optional): list of metric ids to include in desired order.
             If None, take metrics from sample slice.
+
         Returns:
             report
         """
@@ -401,7 +497,7 @@ class TestBench(SemanticVersionerMixin):
                     class_cds = None
                 else:
                     class_names = self.task.output_schema.features[
-                        list(self.task.output_schema.keys())[0]
+                        list(self.task.output_schema.columns)[0]
                     ].names
                     class_cds = [name[0].upper() for name in class_names]
                 columns.append(ClassDistributionColumn(metric_id, class_cds))
@@ -412,7 +508,7 @@ class TestBench(SemanticVersionerMixin):
         columns.append(NumericColumn("Size"))
 
         category_names = {
-            GENERIC: "Slice",
+            GENERIC: "DataPanel",
             SUBPOPULATION: "SubPop",
             ATTACK: "Attack",
             AUGMENTATION: "Augment",
@@ -469,7 +565,7 @@ class TestBench(SemanticVersionerMixin):
             return
 
         if schema_type == "task":
-            self.slices = {self.task.remap_schema(slice) for slice in self.slices}
+            self._slices = {self.task.remap_schema(sl) for sl in self.slices}
             self.schema_type = schema_type
         elif schema_type == "default":
             # TODO(karan): undo the schema standardization
@@ -490,13 +586,6 @@ class TestBench(SemanticVersionerMixin):
             path: string path to the save directory
 
         Returns: None
-
-        >>> testbench = TestBench(identifier='my-testbench',
-        task=TernaryNaturalLanguageInference())
-        # Save to the current directory
-        >>> testbench.save('.')
-        # Load back the testbench
-        >>> testbench = TestBench.load('my-testbench')
         """
 
         # Path to the save directory
@@ -522,6 +611,10 @@ class TestBench(SemanticVersionerMixin):
                 "task": self.task,
                 "identifier": self.identifier,
                 "dataset_id": self.dataset_id,
+                "slices": [
+                    (sl.identifier, sl.category, sl.lineage, len(sl))
+                    for sl in self.slices
+                ],
             },
             open(str(savedir / "metadata.dill"), "wb"),
         )
@@ -555,6 +648,24 @@ class TestBench(SemanticVersionerMixin):
 
         return testbench_identifiers
 
+    @classmethod
+    def load_metrics(cls, path: str) -> dict:
+        """Load metrics from disk."""
+        # Path to the save directory
+        savedir = pathlib.Path(path)
+
+        # Load metrics
+        return dill.load(open(str(savedir / "metrics.dill"), "rb"))
+
+    @classmethod
+    def load_metadata(cls, path: str) -> dict:
+        """Load metrics from disk."""
+        # Path to the save directory
+        savedir = pathlib.Path(path)
+
+        # Load metrics
+        return dill.load(open(str(savedir / "metadata.dill"), "rb"))
+
     @classmethod
     def load(cls, path: str) -> TestBench:
         """Load a testbench from disk.
@@ -572,7 +683,7 @@ class TestBench(SemanticVersionerMixin):
         slices = []
         for sl_path in tqdm(list((savedir / "slices").glob("*"))):
             try:
-                slices.append(Slice.load_from_disk(str(sl_path)))
+                slices.append(DataPanel.load_from_disk(str(sl_path)))
             except FileNotFoundError:
                 continue
 
@@ -597,15 +708,3 @@ class TestBench(SemanticVersionerMixin):
             testbench._loads_version(f.read())
 
         return testbench
-
-    def make(self, identifier: str):
-        # Resolve the location of the TestBench
-
-        # Pull the TestBench
-        return self.pull(identifier)
-
-    def pull(self, identifier: str):
-        pass
-
-    def publish(self):
-        pass
