@@ -1,10 +1,10 @@
-from typing import Callable, Dict, List, Tuple, Union
+from typing import Callable, List, Tuple, Union
 
 import numpy as np
 
-from robustnessgym.core.cachedops import ScoreOperation
-from robustnessgym.core.dataset import Batch, Dataset
 from robustnessgym.core.identifier import Identifier
+from robustnessgym.core.operation import Operation, lookup
+from robustnessgym.core.slice import SliceDataPanel as DataPanel
 from robustnessgym.slicebuilders.subpopulation import Subpopulation
 
 
@@ -15,7 +15,7 @@ class BinningMixin:
         bin_creation_fn: Callable = None,
         bin_fn: Callable = None,
         *args,
-        **kwargs
+        **kwargs,
     ):
         super(BinningMixin, self).__init__(*args, **kwargs)
 
@@ -55,7 +55,7 @@ class BinningMixin:
         self.right_limits = np.array([interval[1] for interval in self.intervals])
 
     def bin(self, scores: List[Union[int, float]]) -> np.ndarray:
-        # Convert to np.ndarry
+        # Convert to np.ndarray
         scores = np.array(scores)
 
         # Bin the scores
@@ -74,7 +74,7 @@ class ScoreSubpopulation(Subpopulation, BinningMixin):
         bin_creation_fn: Callable = None,
         bin_fn: Callable = None,
         *args,
-        **kwargs
+        **kwargs,
     ):
 
         if not identifiers:
@@ -91,7 +91,9 @@ class ScoreSubpopulation(Subpopulation, BinningMixin):
             else:
                 identifiers = [
                     Identifier(
-                        _name=self.__class__.__name__, gte=interval[0], lte=interval[1]
+                        _name=self.__class__.__name__,
+                        gte=interval[0],
+                        lte=interval[1],
                     )
                     for interval in intervals
                 ]
@@ -111,11 +113,11 @@ class ScoreSubpopulation(Subpopulation, BinningMixin):
 
     def prepare_dataset(
         self,
-        dataset: Dataset,
+        dp: DataPanel,
         columns: List[str],
         batch_size: int = 32,
         *args,
-        **kwargs
+        **kwargs,
     ) -> None:
 
         # First reset the scores
@@ -123,7 +125,7 @@ class ScoreSubpopulation(Subpopulation, BinningMixin):
 
         # Prepare the dataset
         super(ScoreSubpopulation, self).prepare_dataset(
-            dataset=dataset,
+            dp=dp,
             columns=columns,
             batch_size=batch_size,
             *args,
@@ -133,34 +135,45 @@ class ScoreSubpopulation(Subpopulation, BinningMixin):
         # Create the bins
         self.create_bins()
 
-    def prepare_batch(self, batch: Batch, columns: List[str], *args, **kwargs) -> None:
+    def prepare_batch(
+        self,
+        batch: DataPanel,
+        columns: List[str],
+        *args,
+        **kwargs,
+    ) -> None:
 
         # Compute the scores
-        if isinstance(self.score, ScoreOperation):
-            self.scores.extend(self.score.retrieve(batch=batch, columns=columns))
+        if isinstance(self.score, Operation):
+            self.scores.extend(lookup(batch, self.score, columns))
         elif isinstance(self.score, Callable):
-            self.scores.extend(self.score(batch=batch, columns=columns))
+            self.scores.extend(self.score(batch, columns))
         else:
             raise RuntimeError("score function invalid.")
 
     def score(
-        self, batch: Dict[str, List], columns: List[str], *args, **kwargs
+        self,
+        batch: DataPanel,
+        columns: List[str],
+        *args,
+        **kwargs,
     ) -> np.ndarray:
         raise NotImplementedError("Return a vector of float scores for each example.")
 
     def apply(
         self,
-        slice_membership: np.ndarray,
-        batch: Dict[str, List],
+        batch: DataPanel,
         columns: List[str],
+        slice_membership: np.ndarray = None,
         *args,
-        **kwargs
+        **kwargs,
     ) -> np.ndarray:
+
         # Keep track of the score of each example
-        if isinstance(self.score, ScoreOperation):
-            scores = self.score.retrieve(batch=batch, columns=columns)
+        if isinstance(self.score, Operation):
+            scores = lookup(batch, self.score, columns)
         elif isinstance(self.score, Callable):
-            scores = self.score(batch=batch, columns=columns)
+            scores = self.score(batch, columns)
         else:
             raise RuntimeError("score function invalid.")
 
@@ -180,7 +193,7 @@ class MultiScoreSubpopulation(Subpopulation, BinningMixin):
         bin_creation_fn: Callable = None,
         bin_fn: Callable = None,
         *args,
-        **kwargs
+        **kwargs,
     ):
 
         if not identifiers:
@@ -217,11 +230,11 @@ class MultiScoreSubpopulation(Subpopulation, BinningMixin):
 
     def prepare_dataset(
         self,
-        dataset: Dataset,
+        dp: DataPanel,
         columns: List[str],
         batch_size: int = 32,
         *args,
-        **kwargs
+        **kwargs,
     ) -> None:
 
         # First reset the scores
@@ -229,7 +242,7 @@ class MultiScoreSubpopulation(Subpopulation, BinningMixin):
 
         # Prepare the dataset
         super(MultiScoreSubpopulation, self).prepare_dataset(
-            dataset=dataset,
+            dp=dp,
             columns=columns,
             batch_size=batch_size,
         )
@@ -237,12 +250,62 @@ class MultiScoreSubpopulation(Subpopulation, BinningMixin):
         # Create the bins
         self.create_bins()
 
-    def prepare_batch(self, batch: Batch, columns: List[str], *args, **kwargs) -> None:
+    def prepare_batch(
+        self,
+        batch: DataPanel,
+        columns: List[str],
+        *args,
+        **kwargs,
+    ) -> None:
 
         # Compute the scores
-        if isinstance(self.score, ScoreOperation):
-            self.scores.extend(self.score.retrieve(batch=batch, columns=columns))
+        if isinstance(self.score, Operation):
+            self.scores.extend(lookup(batch, self.score, columns))
         elif isinstance(self.score, Callable):
-            self.scores.extend(self.score(batch=batch, columns=columns))
+            self.scores.extend(self.score(batch, columns))
         else:
             raise RuntimeError("score function invalid.")
+
+
+BinarySubpopulation = lambda name, score_fn: ScoreSubpopulation(
+    identifiers=[Identifier(f"No{name}"), Identifier(f"{name}")],
+    intervals=[(0, 0), (1, 1)],
+    score_fn=score_fn,
+)
+
+PercentileSubpopulation = lambda name, score_fn: ScoreSubpopulation(
+    identifiers=[
+        Identifier(f"{name}", gte=f"{gte}%", lte=f"{lte}%")
+        for (gte, lte) in [
+            (0, 5),
+            (0, 10),
+            (0, 20),
+            (20, 40),
+            (40, 60),
+            (60, 80),
+            (80, 100),
+            (90, 100),
+            (95, 100),
+        ]
+    ],
+    intervals=[
+        ("0%", "5%"),
+        ("0%", "10%"),
+        ("0%", "20%"),
+        ("20%", "40%"),
+        ("40%", "60%"),
+        ("60%", "80%"),
+        ("80%", "100%"),
+        ("90%", "100%"),
+        ("95%", "100%"),
+    ],
+    score_fn=score_fn,
+)
+
+IntervalSubpopulation = lambda name, intervals, score_fn: ScoreSubpopulation(
+    identifiers=[
+        Identifier(f"{name}", gte=f"{gte}", lte=f"{lte}") for (gte, lte) in intervals
+    ],
+    intervals=intervals,
+    score_fn=score_fn,
+)
